using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Focas.NET.Wikipediaise {
    [Serializable]
    [XmlRoot("entry")]
    public class WikiEntry{
        private const string _wikipediaUrl="http://en.wikipedia.org/wiki/";

        /// <summary>
        /// The text that will be hyperlinked.
        /// </summary>
        [XmlAttribute("key")]
        public string Key {
            get;
            set;
        }

        [XmlAttribute("noHyperlink")]
        public string noHyperlink {
            get;
            set;
        }

        private string _url=null;
        /// <summary>
        /// The url may be determined in a few ways. The XMl file may contain an <code>url</code> attribute. If no <code>url</code> attribute
        /// is present then the <code>wikipediaEntry</code> attribute is checked. If this exists, then it is appended to the url
        /// for the wikipedia site. If neither of the previous conditions are met, then the key is appended to the url for the
        /// wikipedia site.
        /// </summary>
        [XmlAttribute("url")]
        public string Url {
            set {
                _url = value;
            }
            get {
                if (noHyperlink != null) {
                    return string.Empty;
                }
                if(_url == null) {
                    if(WikipediaEntry != null) {
                        _url = String.Concat(_wikipediaUrl, WikipediaEntry);
                    } else {
                        _url = String.Concat(_wikipediaUrl, Key);
                    }
                }
                return _url;
            }
        }
        /// <summary>
        /// This attribute will become the text displayed when a user hovers over the hyperlink.
        /// It may also be used when creating a table of abbreviations
        /// </summary>
        [XmlAttribute("description")]
        public string Description {
            get;
            set;
        }

        /// <summary>
        /// This is used when the wikipedia page does not have the same name as the acronym
        /// </summary>
        [XmlAttribute("wikipediaEntry")]
        public string WikipediaEntry {
            get;
            set;
        }
    }
}
