using System;
using System.Windows.Forms;
using Microsoft.VisualStudio.Tools.Applications.Runtime;
using Word = Microsoft.Office.Interop.Word;
using Office = Microsoft.Office.Core;
using System.IO;
using System.Xml.Serialization;
using System.Reflection;
using System.Collections.Generic;

namespace Focas.NET.Wikipediaise
{
    public partial class Wikipediaise
    {
        private const string XMLNAMESPACE="http://focas.net.au/schemas/wikipediaise";

        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            ribbon.WikipediaiseEvent += new WikipediaiseRibbon.WikipediaiseEventHandler(OnWikipediaise);
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
        }

        public void OnWikipediaise(WikipediaiseRibbon.WikipediaiseEventType eventType)
        {
            if (eventType.Equals(WikipediaiseRibbon.WikipediaiseEventType.Wikipediaise))
            {
                WikipediaiseDocument();
            }
            else if (eventType.Equals(WikipediaiseRibbon.WikipediaiseEventType.MakeAcronymTable))
            {
                MakeAcronymTable();
            }
            else if (eventType.Equals(WikipediaiseRibbon.WikipediaiseEventType.MakeAcronymTableForAllAcronyms))
            {
                MakeAcronymTableForAllAcronyms();
            }
        }

        private void WikipediaiseDocument()
        {

            WikiEntries wikiEntries = null;
            // Word methods require object references. These are helpers for the execute method of the find object
            object oTrue = true;
            object oFalse = false;
            object missing = Type.Missing;
            object replacementText = string.Empty;
            object replace = Word.WdReplace.wdReplaceNone;
            object wrap = Word.WdFindWrap.wdFindContinue;

            using (Stream s = GetWikipediaEntries())
            {
                XmlSerializer ser = new XmlSerializer(typeof(WikiEntries)); //, XMLNAMESPACE);
                wikiEntries = (WikiEntries)ser.Deserialize(s);
                try
                {
                    foreach (WikiEntry entry in wikiEntries.Entries)
                    {
                        Application.ActiveDocument.Characters[1].Select();
                        Word.Find find = Application.Selection.Find;
                        object key = entry.Key;
                        object url = entry.Url;
                        object description = entry.Description;
                        bool keepGoing = true;
                        if (((string)url).Length > 0) {
                            while (keepGoing) {
                                // loop thru until we have replaced at least one entry. This gets around any situation where the first
                                // instance of a key found is an excluded style. It may appear in text later, where it should be hyperlinked
                                if (find.Execute(ref key, ref oTrue, ref oTrue, ref oFalse, ref oFalse, ref oFalse,
                                                                ref oTrue, ref wrap, ref oFalse, ref replacementText, ref replace, ref oFalse,
                                                                ref oFalse, ref oFalse, ref oFalse)) {//, ref oFalse, ref oFalse, ref oFalse,
                                    //ref oFalse, ref oTrue)) {
                                    // Make usre a previously hyperlinked entry isn't hyperlinked again
                                    if (Application.Selection.Hyperlinks.Count == 0) {
                                        // and ensure this style isn't excluded in the wikipediaise.dic file
                                        if (StyleIsNotToBeExcluded(wikiEntries)) {
                                            Application.Selection.Hyperlinks.Add(Application.Selection.Range, ref url,
                                                ref missing, ref description, ref missing, ref missing);
                                            keepGoing = false;
                                        }
                                    } else {
                                        keepGoing = false;
                                    }
                                } else {
                                    keepGoing = false;
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    // TODO: it would be worthwhile adding error reporting here. Most likely
                    // error would be a bad wikipedia.dic file
                    System.Diagnostics.Debug.WriteLine(ex);
                }
            }
        }

        private void MakeAcronymTable()
        {
            // cache the selection as this will be altered in the process of retrieving the acronyms
            Word.Selection sel = Application.Selection;
            object rangeStart = sel.Range.Start;
            object rangeEnd = sel.Range.End;
            // This method unfortunately modifies the location of the selection. This should be changed as the method should not
            // have an effect on the selection object
            List<WikiEntry> acronymsUsed = GetAcronymsUsedInDocument();
            // return the selection object to where the user had originally selected
            Application.ActiveDocument.Range(ref rangeStart, ref rangeEnd).Select();

            // Set up
            object defaultBehaviour = Word.WdDefaultTableBehavior.wdWord8TableBehavior;
            object autoFitBehaviour = Word.WdAutoFitBehavior.wdAutoFitContent;
            Word.Table table = Application.ActiveDocument.Tables.Add(sel.Range, acronymsUsed.Count + 1, 2,
                ref defaultBehaviour, ref defaultBehaviour);
            table.Rows[1].Cells[1].Range.Text = "Abbreviation";
            table.Rows[1].Cells[1].Range.Font.Bold = 1;
            table.Rows[1].Cells[2].Range.Text = "Description";
            table.Rows[1].Cells[2].Range.Font.Bold = 1;

            for (int x = 0; x < acronymsUsed.Count; x++)
            {
                object url = acronymsUsed[x].Url;
                object description = acronymsUsed[x].Description;
                table.Rows[x + 2].Cells[1].Range.Text = acronymsUsed[x].Key;
                table.Rows[x + 2].Cells[2].Range.Text = acronymsUsed[x].Description;
                if (acronymsUsed[x].noHyperlink == null) {
                    table.Rows[x + 2].Cells[2].Range.Hyperlinks.Add(table.Rows[x + 2].Cells[2].Range, ref url,
                                                ref missing, ref description, ref missing, ref missing);
                }

            }
            object tableGrid = "Table Grid";
            object tableLable = Word.WdCaptionLabelID.wdCaptionTable;
            object tableTitle = " Abbreviations used in this document";
            object empty = Type.Missing;
            object oFalse = false;
            object above = Word.WdCaptionPosition.wdCaptionPositionAbove;
            table.set_Style(ref tableGrid);
            table.Select();
            Application.Selection.InsertCaption(ref tableLable, ref tableTitle, ref empty, ref above, ref oFalse);
        }
        private void MakeAcronymTableForAllAcronyms()
        {
            // cache the selection as this will be altered in the process of retrieving the acronyms
            Word.Selection sel = Application.Selection;
            object rangeStart = sel.Range.Start;
            object rangeEnd = sel.Range.End;
            // This method unfortunately modifies the location of the selection. This should be changed as the method should not
            // have an effect on the selection object
            // return the selection object to where the user had originally selected
            Application.ActiveDocument.Range(ref rangeStart, ref rangeEnd).Select();

            // Set up
            object defaultBehaviour = Word.WdDefaultTableBehavior.wdWord8TableBehavior;
            object autoFitBehaviour = Word.WdAutoFitBehavior.wdAutoFitContent;


            using (Stream s = GetWikipediaEntries())
            {
                //XmlSerializer ser = new XmlSerializer(typeof(WikiEntries), XMLNAMESPACE);
                XmlSerializer ser = new XmlSerializer(typeof(WikiEntries));
                WikiEntries wikiEntries = (WikiEntries)ser.Deserialize(s);
                Word.Table table = Application.ActiveDocument.Tables.Add(sel.Range, wikiEntries.Entries.Length + 1, 2,
                    ref defaultBehaviour, ref defaultBehaviour);
                table.Rows[1].Cells[1].Range.Text = "Abbreviation";
                table.Rows[1].Cells[1].Range.Font.Bold = 1;
                table.Rows[1].Cells[2].Range.Text = "Description";
                table.Rows[1].Cells[2].Range.Font.Bold = 1;
                int x = 0;
                foreach (WikiEntry entry in wikiEntries.Entries)
                {
                    object url = entry.Url;
                    object description = entry.Description;
                    table.Rows[x + 2].Cells[1].Range.Text = entry.Key;
                    table.Rows[x + 2].Cells[2].Range.Text = entry.Description;
                    if (entry.noHyperlink == null) {
                        table.Rows[x + 2].Cells[2].Range.Hyperlinks.Add(table.Rows[x + 2].Cells[2].Range, ref url,
                                                    ref missing, ref description, ref missing, ref missing);
                    }
                    x++;
                }
                object tableGrid = "Table Grid";
                object tableLable = Word.WdCaptionLabelID.wdCaptionTable;
                object tableTitle = " Abbreviations defined in wikipedia.dic";
                object empty = Type.Missing;
                object oFalse = false;
                object above = Word.WdCaptionPosition.wdCaptionPositionAbove;
                table.set_Style(ref tableGrid);
                table.Select();
                Application.Selection.InsertCaption(ref tableLable, ref tableTitle, ref empty, ref above, ref oFalse);

            }
        }

        private List<WikiEntry> GetAcronymsUsedInDocument()
        {
            WikiEntries wikiEntries = null;
            List<WikiEntry> acronymsUsed = new List<WikiEntry>();

            // Word methods require object references. These are helpers for the execute method of the find object
            object oTrue = true;
            object oFalse = false;
            object missing = Type.Missing;
            object replacementText = string.Empty;
            object replace = Word.WdReplace.wdReplaceNone;
            object wrap = Word.WdFindWrap.wdFindContinue;

            using (Stream s = GetWikipediaEntries())
            {
                XmlSerializer ser = new XmlSerializer(typeof(WikiEntries));
               // XmlSerializer ser = new XmlSerializer(typeof(WikiEntries), XMLNAMESPACE);
                wikiEntries = (WikiEntries)ser.Deserialize(s);
                foreach (WikiEntry entry in wikiEntries.Entries)
                {
                    Application.ActiveDocument.Characters[1].Select();
                    Word.Find find = Application.Selection.Find;
                    object key = entry.Key;
                    object url = entry.Url;
                    object description = entry.Description;
                    if (find.Execute(ref key, ref oTrue, ref oTrue, ref oFalse, ref oFalse, ref oFalse,
                                                    ref oTrue, ref wrap, ref oFalse, ref replacementText, ref replace, ref oFalse,
                                                    ref oFalse, ref oFalse, ref oFalse))
                    {//, ref oFalse, ref oFalse, ref oFalse,

                        acronymsUsed.Add(entry);
                    }
                }
            }
            return acronymsUsed;
        }

        private bool StyleIsNotToBeExcluded(WikiEntries wikiEntries)
        {
            bool include = true;
            Word.Style style = (Word.Style)Application.Selection.get_Style();
            foreach (string styleToExclude in wikiEntries.StylesToExclude)
            {
                if (style.NameLocal == styleToExclude)
                {
                    include = false;
                    break;
                }
            }
            return include;
        }

        private Stream GetWikipediaEntries()
        {
            // Create an XML file in the My Documents folder called "wikipedia.doc" to customize your own
            // entries. Otherwise the default one will be loaded from the assembly resources.
            String myDocuments = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string wikipediaFile = Path.Combine(myDocuments, "wikipedia.dic");
            if (File.Exists(wikipediaFile))
            {
                return File.Open(wikipediaFile, FileMode.Open, FileAccess.Read);
            }
            else
            {
                return GetResourceStream("Focas.NET.Wikipediaise.resources.wikipedia.xml");
            }
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion
        private Stream GetResourceStream(string resourceName)
        {
            Assembly asm = Assembly.GetExecutingAssembly();
            string[] resourceNames = asm.GetManifestResourceNames();
            for (int i = 0; i < resourceNames.Length; ++i)
            {
                if (string.Compare(resourceName, resourceNames[i], StringComparison.OrdinalIgnoreCase) == 0)
                {
                    return asm.GetManifestResourceStream(resourceNames[i]);
                }
            }
            return null;
        }
    }
}
