using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Office = Microsoft.Office.Core;
using System.Drawing;

namespace Focas.NET.Wikipediaise {
    // TODO:
    // This is an override of the RequestService method in the ThisAddIn class.
    // To hook up your custom ribbon uncomment this code.
    public partial class Wikipediaise {
        private WikipediaiseRibbon ribbon;

        protected override object RequestService(Guid serviceGuid) {
            if(serviceGuid == typeof(Office.IRibbonExtensibility).GUID) {
                if(ribbon == null)
                    ribbon = new WikipediaiseRibbon();
                return ribbon;
            }

            return base.RequestService(serviceGuid);
        }
    }

    [ComVisible(true)]
    public class WikipediaiseRibbon : Office.IRibbonExtensibility {
        public enum WikipediaiseEventType {
            Wikipediaise,
            MakeAcronymTable,
            MakeAcronymTableForAllAcronyms
        }

        private Office.IRibbonUI ribbon;
        public delegate void WikipediaiseEventHandler(WikipediaiseEventType eventType);
        public event WikipediaiseEventHandler WikipediaiseEvent; 

        public WikipediaiseRibbon() {
        }

        public void OnWikipediaise(Office.IRibbonControl control) {
            WikipediaiseEventType eventType = WikipediaiseEventType.Wikipediaise;

            switch(control.Id) {
                case "Focas.NET.Wikipediaise.Wikipediaise":
                    break;
                case "Focas.NET.Wikipediaise.MakeAbbreviationTable":
                    eventType = WikipediaiseEventType.MakeAcronymTable;
                    break;
                case "Focas.NET.Wikipediaise.MakeAcronymTableForAllAcronyms":
                    eventType = WikipediaiseEventType.MakeAcronymTableForAllAcronyms;
                    break;
            }
            if(WikipediaiseEvent != null) {
                try {
                    WikipediaiseEvent(eventType);
                } catch(Exception ex) {
                    MessageBox.Show("There is an issue with the wikipedia.dic file. Please check it out\n" + ex.Message);
                    //TODO: Could log something here, but it is important to capture errors to prevent
                    // client code with errors from breaking the add-in
                }
            }
        }

        #region IRibbonExtensibility Members

        public string GetCustomUI(string ribbonID) {
            return GetResourceText("Focas.NET.Wikipediaise.WikipediaiseRibbon.xml");
        }
        public stdole.IPictureDisp ButtonImage(Office.IRibbonControl control) {
            stdole.IPictureDisp pictureDisp = null;
            Assembly asm = Assembly.GetExecutingAssembly();
            string resourceId = string.Empty;
            switch(control.Id) {
                case "Focas.NET.Wikipediaise.Wikipediaise":
                    resourceId = "Focas.NET.Wikipediaise.resources.book_add.png";
                    break;
                case "Focas.NET.Wikipediaise.MakeAbbreviationTable":
                    resourceId = "Focas.NET.Wikipediaise.resources.table_add.png";
                    break;
                case "Focas.NET.Wikipediaise.MakeAcronymTableForAllAcronyms":
                    resourceId = "Focas.NET.Wikipediaise.resources.application_cascade.png";
                    break;
            }
            using(Stream s = asm.GetManifestResourceStream(resourceId)) {
                pictureDisp =
                   ImageConverter.Convert(Image.FromStream(s));
            }
            return pictureDisp;
        }
        #endregion

        #region Ribbon Callbacks

        public void OnLoad(Office.IRibbonUI ribbonUI) {
            this.ribbon = ribbonUI;
        }

        public void OnToggleButton1(Office.IRibbonControl control, bool isPressed) {
            if(isPressed)
                MessageBox.Show("Pressed");
            else
                MessageBox.Show("Released");
        }

        #endregion

        #region Helpers

        private static string GetResourceText(string resourceName) {
            Assembly asm = Assembly.GetExecutingAssembly();
            string[] resourceNames = asm.GetManifestResourceNames();
            for(int i = 0; i < resourceNames.Length; ++i) {
                if(string.Compare(resourceName, resourceNames[i], StringComparison.OrdinalIgnoreCase) == 0) {
                    using(StreamReader resourceReader = new StreamReader(asm.GetManifestResourceStream(resourceNames[i]))) {
                        if(resourceReader != null) {
                            return resourceReader.ReadToEnd();
                        }
                    }
                }
            }
            return null;
        }

        #endregion
    }
    internal class ImageConverter : System.Windows.Forms.AxHost {
        private ImageConverter()
            : base(null) {
        }

        public static stdole.IPictureDisp Convert(System.Drawing.Image image) {
            return (stdole.IPictureDisp)
                AxHost.GetIPictureDispFromPicture(image);
        }
    }
}
